#ifndef EXPERIMENT_H
#define EXPERIMENT_H

#include<string>
#include<vector>

//------------------------------------------------------------------------------------------------------

/**Run one simulation and write a two .csv file, matrix with the matrix of opinion at the last step, and evolution with the opinions depending of the time
    * @param nbAgent the number of agents in the simulation
    * @param nbGroups the number of groups in the simulation
    * @param sigma defines the shape of the propagation function (the more sigma is small, the more the function is tilted)
    * @param mu the grouping factor (the more mu is high, the more the opinions of a same group are regrouping at the end of each step)
    * @param delta the amplitude of the uniformly distributed errors in the reception of the opinions
    * @param gossip the number of gossip in each interaction
    * @param nbStep the number of steps to run in the simulation (one step is nbAgents interactions)
*/
void get_evolution_1rep(unsigned int nbAgents, unsigned int nbGroups, float sigma,float mu, float delta, unsigned int gossip, unsigned int nbStep);

/**Run nbReplica simulations and write the average opinions depending of time on a .csv file
    * @param initialRange the initial difference of reputation between the top and the bottom group
    * @param nbAgent the number of agents in the simulation
    * @param nbGroups the number of groups in the simulation
    * @param sigma defines the shape of the propagation function (the more sigma is small, the more the function is tilted)
    * @param mu the grouping factor (the more mu is high, the more the opinions of a same group are regrouping at the end of each step)
    * @param delta the amplitude of the uniformly distributed errors in the reception of the opinions
    * @param gossip the number of gossip in each interaction
    * @param nbReplica the number of replica of the simulations with the same parameter
    * @param nbInteractions the number of interactions in each simulation
*/
void changeOfOpinionPerGroup(float initialRange,unsigned int nbAgents, unsigned int nbGroups, float sigma,float mu, float delta, unsigned int gossip,
                             unsigned int nbReplica,unsigned int nbInteractions);


/**Run the moment approximation of the model and write the opinions depending of time on a .csv file
    * @param initialRange the initial difference of reputation between the top and the bottom group
    * @param nbAgent the number of agents in the simulation
    * @param nbGroups the number of groups in the simulation
    * @param sigma defines the shape of the propagation function (the more sigma is small, the more the function is tilted)
    * @param mu the grouping factor (the more mu is high, the more the opinions of a same group are regrouping at the end of each step)
    * @param delta the amplitude of the uniformly distributed errors in the reception of the opinions
    * @param gossip the number of gossip in each interaction
    * @param nbInteractions the number of interactions in each simulation
*/
void getApproxEvolution(double initialRange,unsigned int nbAgents, unsigned int nbGroups, double sigma,double mu, double delta, unsigned int gossip,
                             unsigned int nbInteractions);

/**explore the slope for several initial difference of opinion (from 0 to 2) and write the slopes on a .csv file
    * @param filename the name of the output file
    * @param nbAgent the number of agents in the simulation
    * @param nbGroups the number of groups in the simulation
    * @param sigma defines the shape of the propagation function (the more sigma is small, the more the function is tilted)
    * @param mu the grouping factor (the more mu is high, the more the opinions of a same group are regrouping at the end of each step)
    * @param delta the amplitude of the uniformly distributed errors in the reception of the opinions
    * @param gossip the number of gossip in each interaction
    * @param nbInteractions the number of interactions in each simulation
*/
void exploreSlope(std::string fileName,unsigned int nbAgents, unsigned int nbGroups, double sigma,double mu, double delta, unsigned int gossip,unsigned int nbInteractions);

#endif // EXPERIMENT_H
