#ifndef APPROXIMATION_H
#define APPROXIMATION_H

#include "Utilitary.h"
#include <vector>
#include <string>

//--------------------------------------------------CLASS

class Approximation
{
public:
//ATTRIBUTS

//Parameters
  double                      delta;                          /**< the amplitude of the uniformly distributed errors in the reception of the opinions */
  double                      sigma;                          /**< defines the shape of the propagation function (the more sigma is small, the more the function is tilted) */
  unsigned int                gossip;                         /**< the number of agents subject of gossip in each pair interaction */
  double                      mu;
  unsigned int                nbAgents;
  unsigned int                nbGroups;

//Informations
  unsigned int                nbStep;                         /**< number of steps done yet (1 pair interaction per agent per step) */
  std::vector<IdGroup>        groupOf;
  std::vector<double>         startReputation;                /**< the starting reputation of agents*/
  NVector<2, double>          equis;
  NVector<3, double>          effectByGroup;
  NVector<3, double>          positiveBias;
  NVector<3, double>          negativeBias;
  NVector<3, double>          gossipBias;
  NVector<3, double>          opinions;                       /**< all the average opinions at different time step opinions[t][i][j] = the change of opinion of agents of group i about agents of group jj
                                                                    if j is equal to Ng, then we talk about the self-opinion of i*/


  //use for approximation
  NVector<2, double>          influences;                     /**< the influences of agents : influences[i][j]=starting influence of j on i*/
  NVector<2, double>          dInfluences;                    /**< the derivative of influences : dInfluences[i][j] = derivative of influences of j on i at the start*/
  double                      noise;

  //products approximation
  NVector<1, double>          xSI2;
  NVector<2, double>          xJI2;
  NVector<2, double>          xSIJI;
  NVector<3, double>          xJIKI;
  NVector<2, double>          xSISJ;
  NVector<2, double>          xSIIJ;
  NVector<2, double>          xJIIJ;
  NVector<3, double>          xSIJK;
  NVector<3, double>          xJIJK;
  NVector<3, double>          xJIIK;
  NVector<4, double>          xJIKL;

  Approximation(double delta_,double sigma_, double mu_, unsigned int nbAgents_,unsigned int nbGossips_,std::vector<double> startingRepGroups);

  void averageStep();
  void updateInflus();
  void writeOpinions(std::string fileName);

  //getters
  double getNoise2();
  double getTrueOpinion(IdGroup i, IdGroup j);
  double getTrueSelfOpinion(IdGroup i);
};

#endif // APPROXIMATION_H
