#ifndef LEVIATHAN_H
#define LEVIATHAN_H

#include"Utilitary.h"

#include <unordered_map>
#include <stdlib.h>
#include <list>
#include <string>
#include <fstream>
#include <iomanip>

class Leviathan;


//--------------------------------------------------CLASS

class Agent
{
public:
//ATTRIBUTS

    Leviathan *                 dady;                       //Pointer to Leviathan that contain all agents
    IdGroup                     group;                      //Group of the agent
    IdAgent                     id;                         //id of the agent
    std::vector<float>          opinions;                   //opinions about all agents
    std::vector<float>          deltaOpinions;              //next modifications of opinions about agents

    std::vector<std::vector<IdAgent>>        gossipAbout;   //gossipAbout[group of other]=list of possible gossip of agent if meeting other
    std::vector<IdAgent>                      listGossip;   //list the target of the gossips for the step

    IdAgent                     directInteraction;          //the id of the other agent in direct interaction for the step
    unsigned int                countGossipAbout;           //count the number of gossip down for the step
    float                       influenceOther;             //influence (p) of the other (in direct interaction) on the agent

    //ruling the slope of the logistic function determining the propagation coefficients.
    float                       sigma;

//METHODS
    inline          Agent(Leviathan * father,IdGroup group_,IdAgent agent_,float sigma_):
                        dady(father),group(group_),id(agent_),countGossipAbout(0),sigma(sigma_){};

    void            init(unsigned int nbAgents, unsigned int nbGroups);     //called in dady->init()

//RUN Function
    IdAgent         gossipToWho();                  //return an agent to talk about as gossip
    void            updateOpinions();               //update the opinions about agents and groups with the deltas

//getters
    inline float    getOpinion(IdAgent other){return opinions[other];};                 //the opinion of agent about other
};

class Leviathan
{
public:
//alea
    std::mt19937                        fixe_g_m_t_generator;
    std::uniform_int_distribution<int>  getRandomAgent_;
//ATTRIBUTES

//Parameters
                                //the noise in the influence. Correspond to a uniform random number between -noise and noise.
    float                       noise;
                                //the number of gossip in the influence. Each agent talk about that number of agents in one interaction.
    unsigned int                gossip;
                                //the factor of the change of opinion about group
    float                       regroupFactor;

//Informations
    unsigned int                nbStep;                 //number of steps done in the simulation
    std::vector<Agent>          agents;                 //the agents in the simulations
    unsigned int                nbGroup;                //the number of groups in the simulation
    bool                        isInitialized;

//Utilitary
    std::vector<float>          startingOpinionsByGroup;
    std::uniform_real_distribution<float> g_noise;      //random generator for the noise

//Interactions
    bool                        ignoreBorder;
    bool                        oneInterractionParStep;

//INITIALIZATION
    Leviathan(float noise_, unsigned int gossip_,float regroupFactor);
    ~Leviathan();
    void addGroup(unsigned int numberAgents,float sigma_);
    void init(std::vector<float> baseGroupsOpinions=std::vector<float>(),bool useTheSeed=false);
    void resetOpinions();

//RUN
    void step();
    void selectCoupleForInterraction(IdAgent & Ai,IdAgent & Aj);
    void directInteraction(IdAgent Ai,IdAgent Aj);
    void gossipInteraction(IdAgent Ai,IdAgent Aj);
    void oneGossip(IdAgent Ai,IdAgent Aj);
    void regroupOpinions();


    void    writeAllInfos(std::string file_name, unsigned int mode=0);
    void    writeOpinions(std::string fileName);

    void    reduceMinMaxDistance(IdGroup from,IdGroup to,float reduceFactor);
    float                       borne(float value);

//getters
    inline float                getOpinion(IdAgent i,IdAgent j){return agents[i].getOpinion(j);};
    inline unsigned int         getNbAgents(){return agents.size();};
    inline unsigned int         getNbGroups(){return nbGroup;};
    inline IdAgent              getRandomAgent(){return getRandomAgent_(fixe_g_m_t_generator);};
};

#endif // LEVIATHAN_H
