#include "Leviathan.h"
#include "Experiment.h"
#include"LevApproximation.h"

#include <fstream>
#include <iostream>
#include <sstream>

template <typename T>
std::string to_string_with_precision(const T a_value, const int n = 6)
{
    std::ostringstream out;
    out.precision(n);
    out << std::fixed << a_value;
    return out.str();
}


void get_evolution_1rep(unsigned int nbAgents, unsigned int nbGroups, float sigma,float mu, float delta, unsigned int gossip, unsigned int nbStep){
    std::string after="Na"+std::to_string(nbAgents)+"_Ng"+std::to_string(nbGroups)+"_k"+std::to_string(gossip)+"_sig"+to_string_with_precision(mu,2)+"_d"+std::to_string(delta)+"_mu"+to_string_with_precision(mu,6)+".csv";
    std::string outputM="matrix_" + after;
    std::string outputE="evolution_" + after;
    std::ofstream myfileE;
    myfileE.open(outputE);
    unsigned int Ng=nbAgents/nbGroups;
    unsigned int frequence_print=10;

    myfileE<<"step,idAgent,idGroup,mu,sigma,noise_intensity,gossip";
    for(IdAgent i=0;i<nbAgents;i++){
      myfileE<<","<<i;
    }
    myfileE<<std::endl;

    Leviathan lev(delta,gossip,mu);
    for(IdGroup i=0;i<nbGroups;i++){
      lev.addGroup(Ng,sigma);
    }
    lev.init();
    for(unsigned int t=0;t<nbStep;t++){
      if(t%1000==0)
        std::cout<<t<<std::endl;
      lev.step();
      lev.regroupOpinions();
      if(t%frequence_print==0){
        for(IdAgent i=0;i<nbAgents;i++){
          myfileE<<t<<","<<i<<","<<lev.agents[i].group<<","<<mu<<","<<sigma<<","<<delta<<","<<gossip;
          for(IdAgent j=0;j<nbAgents;j++){
            myfileE<<","<<lev.getOpinion(i,j);
          }
          myfileE<<std::endl;
        }
      }
    }
    myfileE.close();

    std::ofstream myfileM;
    myfileM.open(outputM);
    myfileM<<"Run,population,leftToUse,mu,sigma,noise_intensity,gossip";
    for(IdAgent i=0;i<nbAgents;i++){
      myfileM<<","<<"OPI"<<i;
    }
    myfileM<<std::endl;

    for(IdAgent i=0;i<nbAgents;i++){
      myfileM<<nbStep<<","<<nbAgents<<","<<"0"<<","<<mu<<","<<sigma<<","<<delta<<","<<gossip;
      for(IdAgent j=0;j<nbAgents;j++){
        myfileM<<","<<lev.getOpinion(i,j);
      }
      myfileM<<std::endl;
    }
    myfileM.close();
}

void changeOfOpinionPerGroup(float initialRange,unsigned int nbAgents, unsigned int nbGroups, float sigma,float mu, float delta, unsigned int gossip,
                             unsigned int nbReplica,unsigned int nbInteractions){
  std::ofstream myfile;
  std::string file_name="simulation_initRange=" +to_string_with_precision(initialRange, 2) +"_sig="+to_string_with_precision(sigma, 2)+
    "_d="+to_string_with_precision(delta, 2)+"_mu="+to_string_with_precision(mu, 6)+"_k="+std::to_string(gossip)+"_Na="+std::to_string(nbAgents)+"_Ng="+std::to_string(nbGroups)+"_nbRep="+std::to_string(nbReplica)+
    "_nbInterac="+std::to_string(nbInteractions)+".csv";
  float Ng=nbAgents/nbGroups;
  myfile.open(file_name);
  Leviathan lev(delta,gossip,mu);
  for(unsigned int g=0;g<nbGroups;g++)
    lev.addGroup(nbAgents/nbGroups,sigma);
  lev.oneInterractionParStep=true;
  lev.ignoreBorder=true;
  std::vector<float> baseOpinions(nbAgents,0);
  std::vector<float> baseGroupsOpinions(nbGroups,0);
  float op=-initialRange/2.;
  for(float o=0;o<nbGroups;o++){
    baseGroupsOpinions[o]=op;
    if(nbGroups>1)
      op+=initialRange/(nbGroups-1);
  }
  for(IdAgent i=0;i<nbAgents;i++){
    baseOpinions[i]=baseGroupsOpinions[i/Ng];
  }
  lev.init(baseGroupsOpinions);

  NVector<2, float> sum_Xiiii(nbGroups,nbInteractions,0);
  NVector<3, float> sum_Xjiji(nbGroups,nbGroups,nbInteractions,0);
  NVector<3, float> sum_Xiiji(nbGroups,nbGroups,nbInteractions,0);
  NVector<3, float> sum_Xijji(nbGroups,nbGroups,nbInteractions,0);
  NVector<3, float> sum_Xiiij(nbGroups,nbGroups,nbInteractions,0);
  NVector<3, float> sum_Xiijj(nbGroups,nbGroups,nbInteractions,0);

  NVector<4, float> sum_Xjiki(nbGroups,nbGroups,nbGroups,nbInteractions,0);
  NVector<4, float> sum_Xiijp(nbGroups,nbGroups,nbGroups,nbInteractions,0);
  NVector<4, float> sum_Xijjp(nbGroups,nbGroups,nbGroups,nbInteractions,0);
  NVector<4, float> sum_Xijip(nbGroups,nbGroups,nbGroups,nbInteractions,0);

  NVector<2, float> sum_Xsi(nbGroups,nbInteractions,0);
  NVector<3, float> sum_Xji(nbGroups,nbGroups,nbInteractions,0);
  float nb;
  for(unsigned int r=0;r<nbReplica;r++){
    if(r%10==0)
      std::cout<<r<<std::endl;
    lev.resetOpinions();
    for(unsigned int i=0;i<nbInteractions;i++){
      lev.step();
      for(IdGroup g1=0;g1<nbGroups;g1++){
        nb=0;
        float somm_Xsi=0;
        float somm_Xiiii=0;
        for(IdAgent s1=0;s1<nbAgents;s1++){
          if(lev.agents[s1].group==g1){
            somm_Xiiii+=(lev.getOpinion(s1,s1)-baseOpinions[s1])*(lev.getOpinion(s1,s1)-baseOpinions[s1]);
            somm_Xsi+=(lev.getOpinion(s1,s1)-baseOpinions[s1]);
            nb++;
          }
        }
        sum_Xiiii[g1][i]+=somm_Xiiii/nb;
        sum_Xsi[g1][i]+=somm_Xsi/nb;;

        for(IdGroup g2=0;g2<nbGroups;g2++){
          float somm_Xji=0;
          float somm_Xiijj=0;
          float somm_Xjiji=0;
          float somm_Xiiji=0;
          float somm_Xiiij=0;
          float somm_Xijji=0;
          nb=0;
          for(IdAgent s1=0;s1<nbAgents;s1++){
            if(lev.agents[s1].group==g1){
              for(IdAgent s2=0;s2<nbAgents;s2++){
                if(s1!=s2 && lev.agents[s2].group==g2){
                  somm_Xji+=lev.getOpinion(s2,s1)-baseOpinions[s1];
                  somm_Xiijj+=(lev.getOpinion(s1,s1)-baseOpinions[s1])*(lev.getOpinion(s2,s2)-baseOpinions[s2]);
                  somm_Xjiji+=(lev.getOpinion(s2,s1)-baseOpinions[s1])*(lev.getOpinion(s2,s1)-baseOpinions[s1]);
                  somm_Xiiji+=(lev.getOpinion(s1,s1)-baseOpinions[s1])*(lev.getOpinion(s2,s1)-baseOpinions[s1]);
                  somm_Xiiij+=(lev.getOpinion(s1,s1)-baseOpinions[s1])*(lev.getOpinion(s1,s2)-baseOpinions[s2]);
                  somm_Xijji+=(lev.getOpinion(s1,s2)-baseOpinions[s2])*(lev.getOpinion(s2,s1)-baseOpinions[s1]);
                  nb++;
                }

              }
            }
          }
          sum_Xji[g1][g2][i]+=somm_Xji/nb;
          sum_Xiijj[g1][g2][i]+=somm_Xiijj/nb;
          sum_Xjiji[g1][g2][i]+=somm_Xjiji/nb;
          sum_Xiiji[g1][g2][i]+=somm_Xiiji/nb;
          sum_Xiiij[g1][g2][i]+=somm_Xiiij/nb;
          sum_Xijji[g1][g2][i]+=somm_Xijji/nb;
          if(r%nbAgents==0){
            for(IdGroup g3=0;g3<nbGroups;g3++){
              float somm_Xjiki=0;
              float somm_Xiijp=0;
              float somm_Xijjp=0;
              float somm_Xijip=0;
              nb=0;
              for(IdAgent s1=0;s1<nbAgents;s1++){
                if(lev.agents[s1].group==g1){
                  for(IdAgent s2=0;s2<nbAgents;s2++){
                    if(s1!=s2 && lev.agents[s2].group==g2){
                      for(IdAgent s3=0;s3<nbAgents;s3++){
                        if(s1!=s3 && s2!=s3 && lev.agents[s3].group==g3){
                          somm_Xjiki+=(lev.getOpinion(s2,s1)-baseOpinions[s1])*(lev.getOpinion(s3,s1)-baseOpinions[s1]);
                          somm_Xiijp+=(lev.getOpinion(s1,s1)-baseOpinions[s1])*(lev.getOpinion(s2,s3)-baseOpinions[s3]);
                          somm_Xijjp+=(lev.getOpinion(s1,s2)-baseOpinions[s2])*(lev.getOpinion(s2,s3)-baseOpinions[s3]);
                          somm_Xijip+=(lev.getOpinion(s1,s2)-baseOpinions[s2])*(lev.getOpinion(s1,s3)-baseOpinions[s3]);
                          nb++;
                        }
                      }
                    }
                  }
                }
              }
              sum_Xjiki[g1][g2][g3][i]+=somm_Xjiki/nb;
              sum_Xiijp[g1][g2][g3][i]+=somm_Xiijp/nb;
              sum_Xijjp[g1][g2][g3][i]+=somm_Xijjp/nb;
              sum_Xijip[g1][g2][g3][i]+=somm_Xijip/nb;
            }
          }
        }
      }
    }
  }
  myfile<<"step,gi,gj,gk,gl,rep(i),rep(j),rep(k),rep(l),xSI,xJI,xiiii,xjiji,xiiji,xjiki,xiijj,xjiij,xiiij,xiijp,xijjp,xijip";
  myfile<<std::endl;
  for(IdGroup a1=0;a1<nbGroups;a1++){
    for(IdGroup a2=0;a2<nbGroups;a2++){
      for(IdGroup a3=0;a3<nbGroups;a3++){
        for(IdGroup a4=0;a4<nbGroups;a4++){
          for(unsigned int st=0;st<nbInteractions;st++){
            myfile<<st<<","<<a1<<","<<a2<<","<<a3<<","<<a4<<","<<baseOpinions[a1*nbAgents/nbGroups]<<","<<baseOpinions[a2*nbAgents/nbGroups]<<","<<baseOpinions[a3*nbAgents/nbGroups]<<","<<baseOpinions[a4*nbAgents/nbGroups];
            myfile<<","<<sum_Xsi[a1][st]/nbReplica<<","<<sum_Xji[a1][a2][st]/nbReplica;
            myfile<<","<<sum_Xiiii[a1][st]/nbReplica<<","<<sum_Xjiji[a1][a2][st]/nbReplica<<","<<sum_Xiiji[a1][a2][st]/nbReplica<<","<<sum_Xjiki[a1][a2][a3][st]/(nbReplica/nbAgents);
            myfile<<","<<sum_Xiijj[a1][a2][st]/nbReplica<<","<<sum_Xijji[a1][a2][st]/nbReplica<<","<<sum_Xiiij[a1][a2][st]/nbReplica;
            myfile<<","<<sum_Xiijp[a1][a2][a3][st]/(nbReplica/nbAgents)<<","<<sum_Xijjp[a1][a2][a3][st]/(nbReplica/nbAgents)<<","<<sum_Xijip[a1][a2][a3][st]/(nbReplica/nbAgents)<<std::endl;
          }
        }
      }
    }
  }
  myfile.close();
}

void getApproxEvolution(double initialRange,unsigned int nbAgents, unsigned int nbGroups, double sigma,double mu, double delta, unsigned int gossip,
                             unsigned int nbInteractions){
  std::string file_name="approximation_initRange=" +to_string_with_precision(initialRange, 2) +"_sig="+to_string_with_precision(sigma, 2)+
    "_d="+to_string_with_precision(delta, 2)+"_mu="+to_string_with_precision(mu, 6)+"_k="+std::to_string(gossip)+"_Na="+std::to_string(nbAgents)+"_Ng="+std::to_string(nbGroups)+
    "_nbInterac="+std::to_string(nbInteractions)+".csv";
  std::ofstream myfile;
  myfile.open(file_name);
  myfile<<"step,gi,gj,gk,gl,rep(i),rep(j),rep(k),rep(l),xSI,xJI,xiiii,xjiji,xiiji,xjiki,equi,xiijj,xiiij,xjiij,xiijk,xjijk,xjiik,xjikl"<<std::endl;

  std::vector<double> base(nbGroups,0);
  double op=-initialRange/2.;
  for(IdGroup i=0;i<nbGroups;i++){
    base[i]=op;
    op+=initialRange/double(nbGroups-1);
  }
  Approximation app(delta,sigma,mu,nbAgents,gossip,base);
  for(unsigned int t=0;t<nbInteractions;t++){
    app.averageStep();
    for(IdAgent i=0;i<nbGroups;i++){
      for(IdAgent j=0;j<nbGroups;j++){
        for(IdAgent k=0;k<nbGroups;k++){
          for(IdAgent l=0;l<nbGroups;l++){
              myfile<<t<<","<<i<<","<<j<<","<<k<<","<<l<<","<<app.startReputation[i]<<","<<app.startReputation[j]<<","<<app.startReputation[k]<<","<<app.startReputation[l]<<",";
              myfile<<app.opinions[t][i][nbGroups]<<","<<app.opinions[t][j][i];
              myfile<<","<<app.xSI2[i]<<","<<app.xJI2[j][i]<<","<<app.xSIJI[i][j]<<","<<app.xJIKI[j][i][k];
              myfile<<","<<app.equis[t][i];
              myfile<<","<<app.xSISJ[i][j]<<","<<app.xSIIJ[i][j]<<","<<app.xJIIJ[j][i]<<","<<app.xSIJK[i][j][k];
              myfile<<","<<app.xJIJK[j][i][k]<<","<<app.xJIIK[j][i][k]<<","<<app.xJIKL[j][i][k][l]<<std::endl;
          }
        }
      }
    }
  }
  myfile.close();
}

void exploreSlope(std::string fileName,unsigned int nbAgents, unsigned int nbGroups, double sigma,double mu, double delta, unsigned int gossip,unsigned int step){
  std::ofstream myfile;
  myfile.open(fileName);
  myfile<<"diff,g,rep(g),slopeEquilibrium,slope_xSI,slopeFromSelfGroup,slopeFromOtherGroup,positiveBiasIntra,positiveBiasExtra,negativeBiasIntra,negativeBiasExtra,gossipBias"<<std::endl;
  std::vector<double> base(nbGroups,0);
  for(double diff=0;diff<=2.00001;diff+=0.05){
    double op=0;
    for(IdGroup g=0;g<nbGroups;g++){
      base[g]=op;
      op+=diff/double(nbGroups-1);
    }
    Approximation app(delta,sigma,mu,nbAgents,gossip,base);
    for(unsigned int st=0;st<step;st++){
      app.averageStep();
    }
    for(IdGroup i=0;i<nbGroups;i++){
      myfile<<diff<<","<<i<<","<<base[i]<<","<<app.equis[app.nbStep][i]-app.equis[app.nbStep-1][i]<<","<<app.opinions[app.nbStep][i][nbGroups]-app.opinions[app.nbStep-1][i][nbGroups];
      myfile<<","<<app.effectByGroup[app.nbStep][i][i]<<","<<app.effectByGroup[app.nbStep][i][(i+1)%nbGroups];
      myfile<<","<<app.positiveBias[app.nbStep][i][i]<<","<<app.positiveBias[app.nbStep][i][(i+1)%nbGroups]<<","<<app.negativeBias[app.nbStep][i][i]<<","<<app.negativeBias[app.nbStep][i][(i+1)%nbGroups];
      myfile<<","<<app.gossipBias[app.nbStep][i][i]<<std::endl;
    }
  }
}
