#include "LevApproximation.h"
#include <iostream>
#include <fstream>
#include <math.h>

double influenceFunction(double selfOpinion,double opinionOther,double sigma){
  return 1.0/(1.0+exp((selfOpinion-opinionOther)/sigma));
}

float dInfluenceFunction(double selfOpinion,double opinionOther,double sigma){
  double h= influenceFunction(selfOpinion,opinionOther,sigma);
  return (-h*(1-h))/sigma;
}

Approximation::Approximation(double delta_,double sigma_,double mu_, unsigned int nbAgents_,unsigned int nbGossips_,std::vector<double> startingRepGroups):
  delta(delta_),sigma(sigma_),gossip(nbGossips_),mu(mu_),nbAgents(nbAgents_),nbGroups(startingRepGroups.size()),
  nbStep(0),groupOf(nbAgents_,0),startReputation(startingRepGroups.size(),0),equis(200,startingRepGroups.size(),0),
  effectByGroup(200,startingRepGroups.size(),startingRepGroups.size(),0),
  positiveBias(200,startingRepGroups.size(),startingRepGroups.size(),0),negativeBias(200,startingRepGroups.size(),startingRepGroups.size(),0),gossipBias(200,startingRepGroups.size(),startingRepGroups.size(),0),
  opinions(200,startingRepGroups.size(),startingRepGroups.size()+1,0),
  influences(startingRepGroups.size(),startingRepGroups.size(),0),dInfluences(startingRepGroups.size(),startingRepGroups.size(),0),
  xSI2(startingRepGroups.size(),0),xJI2(startingRepGroups.size(),startingRepGroups.size(),0),xSIJI(startingRepGroups.size(),startingRepGroups.size(),0),xJIKI(startingRepGroups.size(),startingRepGroups.size(),startingRepGroups.size(),0),
  xSISJ(startingRepGroups.size(),startingRepGroups.size(),0),xSIIJ(startingRepGroups.size(),startingRepGroups.size(),0),xJIIJ(startingRepGroups.size(),startingRepGroups.size(),0),
  xSIJK(startingRepGroups.size(),startingRepGroups.size(),startingRepGroups.size(),0),xJIJK(startingRepGroups.size(),startingRepGroups.size(),startingRepGroups.size(),0),
  xJIIK(startingRepGroups.size(),startingRepGroups.size(),startingRepGroups.size(),0),
  xJIKL(startingRepGroups.size(),startingRepGroups.size(),startingRepGroups.size(),startingRepGroups.size(),0)
{
  for(unsigned int i=0;i<nbGroups;i++){
    startReputation[i]=startingRepGroups[i];
  }
  for(unsigned int i=0;i<nbAgents;i++){
    groupOf[i]=floor(i/(nbAgents/nbGroups));
  }
  updateInflus();
  noise=pow(delta,2)/3.;
}

void Approximation::updateInflus(){
  for(IdGroup i=0;i<nbGroups;i++){
    for(IdGroup j=0;j<nbGroups;j++){
      dInfluences[i][j]=dInfluenceFunction(getTrueSelfOpinion(i),getTrueOpinion(i,j),sigma);
      //TODO : check this
      //influences[i][j]=influenceFunction(getTrueSelfOpinion(i),getTrueOpinion(i,j),sigma);
      influences[i][j]=influenceFunction(getTrueSelfOpinion(i),getTrueOpinion(i,j),sigma) ;//- dInfluences[i][j]*(opinions[nbStep][i][nbGroups] - opinions[nbStep][i][j]);
    }
  }
}

void Approximation::averageStep(){
  double Na=nbAgents;
  double ng=nbGroups;
  double Ng=double(nbAgents)/double(nbGroups);
  double Nc=Na*(Na-1);
  double Nt=Na*(Na-1)*(Na-2);
  double Dij;
  double Dip,Djp;
  double Dqi,Dqj,Dqp;

  NVector<4, double> newMultis(ng,ng+1,ng,ng+1,0);

  //extend the size of the vector if it is necessary (or close to be)
  if((nbStep+5)>opinions.size()){
    opinions.push_back(NVector<2,double>(ng,ng+1,0));
    equis.push_back(NVector<1,double>(ng,0));
    effectByGroup.push_back(NVector<2,double>(ng,ng,0));
    positiveBias.push_back(NVector<2,double>(ng,ng,0));
    negativeBias.push_back(NVector<2,double>(ng,ng,0));
    gossipBias.push_back(NVector<2,double>(ng,ng,0));
  }


  NVector<2, double>           new_xSI2(2,ng,0);
  NVector<3, double>           new_xJI2(2,ng,ng,0);
  NVector<3, double>           new_xSIJI(2,ng,ng,0);
  NVector<4, double>           new_xJIKI(2,ng,ng,ng,0);

  NVector<3, double>           new_xSISJ(2,ng,ng,0);
  NVector<3, double>           new_xSIIJ(2,ng,ng,0);
  NVector<3, double>           new_xJIIJ(2,ng,ng,0);
  NVector<4, double>           new_xSIJK(2,ng,ng,ng,0);
  NVector<4, double>           new_xJIJK(2,ng,ng,ng,0);
  NVector<4, double>           new_xJIIK(2,ng,ng,ng,0);
  NVector<5, double>           new_xJIKL(2,ng,ng,ng,ng,0);

  double s_si,s_ji,s_si2,s_siji,s_siji_g,s_ji2,s_jiki;
  double s_sisj,s_siij,s_sijk;
  double g_jikl,g_sijk,g_jijk,g_jiik,g_jiij,g_siij;
  for(IdGroup i=0;i<ng;i++){
    s_si=0;s_si2=0;
    for(IdGroup j=0;j<ng;j++){
      Dij=(i==j);
      s_siji=0;
      s_ji2=0;
      s_siji_g=0;
      s_ji=0;
      s_sisj=0;
      s_siij=0;
      g_jiij=0;
      g_siij=0;
      for(IdGroup p=0;p<ng;p++){
        Dip=(i==p);
        Djp=(j==p);

        s_jiki=0;
        s_sijk=0;
        g_sijk=0;
        g_jijk=0;
        g_jiik=0;
        for(IdGroup q=0;q<ng;q++){
          Dqi=(i==q);
          Dqj=(j==q);
          Dqp=(p==q);
          s_jiki+=(Ng-Dqi-Dqj-Dqp)*( influences[j][q]*(xJIKI[q][i][p]-xJIKI[j][i][p]) + influences[p][q]*(xJIKI[q][i][j]-xJIKI[j][i][p]) );

          s_sijk+=(Ng-Dqi)*influences[i][q]*(xJIKL[q][i][j][p] - xSIJK[i][j][p]);

          g_sijk+=(Ng-Dqj-Dqp)*influences[j][q]*(xSIJK[i][q][p] - xSIJK[i][j][p]);

          g_jijk+=(Ng-Dqj-Dqp)*influences[j][q]*(xJIKL[j][i][q][p] - xJIJK[j][i][p])
                  +(Ng-Dqj-Dqi)*influences[j][q]*(xJIKL[q][i][j][p] - xJIJK[j][i][p]);

          g_jiik+=(Ng-Dqi-Dqp)*influences[p][q]*(xJIIK[q][i][j] - xJIIK[j][i][p])
                  +(Ng-Dqi-Dqj)*influences[i][q]*(xJIKL[p][i][q][j] - xJIIK[j][i][p]);

          g_jikl=0;
          for(IdGroup r=0;r<ng;r++){
            g_jikl+=(Ng-(r==p)-(r==q))*influences[p][r]*(xJIKL[j][i][r][q]-xJIKL[j][i][p][q])
                    +(Ng-(r==j)-(r==i))*influences[p][r]*(xJIKL[r][i][p][q]-xJIKL[j][i][p][q]);
          }
          new_xJIKL[0][j][i][p][q] = xJIKL[j][i][p][q] + 2./Nc*(influences[j][i]*(xSIJK[i][p][q] - xJIKL[j][i][p][q]) + influences[p][q]*(xSIJK[q][j][i] - xJIKL[j][i][p][q]))
                                                        +gossip*2./Nt*g_jikl;
        }

        s_ji += (Ng-Dip-Djp)*influences[j][p]*(opinions[nbStep][p][i] - opinions[nbStep][j][i]);

        s_ji2 += (Ng-Dip-Djp)*( pow(1-influences[j][p],2)*xJI2[j][i] + pow(influences[j][p],2)*xJI2[p][i] + 2*(1-influences[j][p])*influences[j][p]*xJIKI[j][i][p] + pow(influences[j][p],2)*getNoise2() - xJI2[j][i] );

        s_siji_g += (Ng-Dip-Djp)*influences[j][p]*( xSIJI[i][p] - xSIJI[i][j]);

        s_siji+=(Ng-Dip-Djp)*influences[i][p]*(xJIKI[j][i][p] - xSIJI[i][j]);

        s_sisj+=(Ng-Dip-Djp)*(influences[i][p]*(xSIJK[j][p][i] - xSISJ[i][j]) + influences[j][p]*(xSIJK[i][p][j] - xSISJ[i][j]));
//old
        s_siij+=(Ng-Dip-Djp)*influences[i][p]*(xJIIK[p][i][j] - xSIIJ[i][j]);
//test
//        s_siij+=(Ng-Dip-Djp)*(1-gossip/float(Na-2))*influences[i][p]*(xJIIK[p][i][j] - xSIIJ[i][j]);


        g_jiij+=(Ng-Dip-Djp)*(influences[j][p]*(xJIIK[p][i][j] - xJIIJ[i][j]) + influences[i][p]*(xJIIK[p][j][i] - xJIIJ[i][j]));

//old
        g_siij+=(Ng-Dip-Djp)*influences[i][p]*(xSIJK[i][p][j] - xSIIJ[i][j]);
//test
//        g_siij+=(Ng-Dip-Djp)*(pow(influences[i][p],2)*(xJIJK[p][i][j]-xSIJK[i][p][j]-xJIIK[p][i][j]+xSIIJ[i][j])
//                              +influences[i][p]*xJIIK[p][i][j] + xSIJK[i][p][j]-xSIIJ[i][j]);

        new_xJIKI[0][j][i][p] = xJIKI[j][i][p] + 2./Nc*(influences[p][i]*(xSIJI[i][j]-xJIKI[j][i][p]) + influences[j][i]*(xSIJI[i][p]-xJIKI[j][i][p]))
                                              + gossip*2./Nt*( (1-influences[j][p])*(1-influences[p][j])*xJIKI[j][i][p] + influences[j][p]*influences[p][j]*xJIKI[j][i][p]
                                                              +(1-influences[j][p])*influences[p][j]*xJI2[j][i] + (1-influences[p][j])*influences[j][p]*xJI2[p][i] - xJIKI[j][i][p]
                                                              + s_jiki);

        new_xSIJK[0][i][j][p] =  xSIJK[i][j][p] + 2./Nc*(influences[j][p]*(xSISJ[i][p] - xSIJK[i][j][p])
                                                        +s_sijk)
                                                  +2.*gossip/Nt*g_sijk;

        new_xJIIK[0][j][i][p] = xJIIK[j][i][p] + 2./Nc*(influences[j][i]*(xSIIJ[i][p] - xJIIK[j][i][p]) + influences[i][p]*(xSIJK[p][j][i] - xJIIK[j][i][p]))
                                              +2.*gossip/Nt*g_jiik;
        new_xJIJK[0][j][i][p] = xJIJK[j][i][p] + 2./Nc*(influences[j][i]*(xSIJK[i][j][p] - xJIJK[j][i][p]) + influences[j][p]*(xSIJK[p][j][i] - xJIJK[j][i][p]))
                                                + 2.*gossip/Nt*g_jijk;
      }
      s_si += 2.*(Ng-Dij)/Nc*(influences[i][j]*(opinions[nbStep][j][i] - opinions[nbStep][i][ng]) + dInfluences[i][j]*(xSIJI[i][j] - xSI2[i] + xSIIJ[i][j] - xJIIJ[i][j]));
      s_si2 += 2.*(Ng-Dij)/Nc*(pow(1-influences[i][j],2)*xSI2[i] + pow(influences[i][j],2)*xJI2[j][i] + 2*(1-influences[i][j])*influences[i][j]*xSIJI[i][j]+pow(influences[i][j],2)*getNoise2()-xSI2[i]);



      new_xJI2[0][j][i]=xJI2[j][i] + 2./Nc*(pow(1-influences[j][i],2)*xJI2[j][i] + pow(influences[j][i],2)*xSI2[i] + 2*(1-influences[j][i])*influences[j][i]*xSIJI[i][j] + pow(influences[j][i],2)*getNoise2()-xJI2[j][i] )
                                    +gossip*2./Nt*s_ji2;

      new_xSIJI[0][i][j]=xSIJI[i][j] + 2./Nc*((1-influences[i][j])*(1-influences[j][i])*xSIJI[i][j] + influences[i][j]*influences[j][i]*xSIJI[i][j]
                                                   + (1-influences[i][j])*influences[j][i]*xSI2[i] + (1-influences[j][i])*influences[i][j]*xJI2[j][i]-xSIJI[i][j]
                                                   + s_siji)
                                    + gossip*2./Nt*s_siji_g;

      new_xSISJ[0][i][j]=xSISJ[i][j] + 2./Nc*(influences[i][j]*(xSIIJ[j][i] - xSISJ[i][j]) + influences[j][i]*(xSIIJ[i][j] - xSISJ[i][j])
                                              +influences[i][j]*influences[j][i]*(xJIIJ[j][i] - xSIIJ[i][j] - xSIIJ[j][i] + xSISJ[i][j])
                                              +s_sisj);
//old
      new_xSIIJ[0][i][j]=xSIIJ[i][j] + 2./Nc*(influences[i][j]*(xJIIJ[j][i] + xSISJ[i][j] -2*xSIIJ[i][j])
                                              +influences[i][j]*influences[i][j]*(xSIIJ[j][i] - xJIIJ[j][i] - xSISJ[i][j] + xSIIJ[i][j])
                                              +s_siij)
                                      + 2.*gossip/Nt*g_siij;
//test
//      new_xSIIJ[0][i][j]=xSIIJ[i][j] + 2./Nc*(influences[i][j]*(xJIIJ[j][i] + xSISJ[i][j] -2*xSIIJ[i][j])
//                                              +influences[i][j]*influences[i][j]*(xSIIJ[j][i] - xJIIJ[j][i] - xSISJ[i][j] + xSIIJ[i][j])
//                                              +s_siij)
//                                      + 2.*gossip/Nt*g_siij;

      new_xJIIJ[0][j][i]=xJIIJ[j][i] + 2./Nc*(influences[j][i]*(xSIIJ[i][j] - xJIIJ[j][i]) + influences[i][j]*(xSIIJ[j][i] - xJIIJ[i][j])
                                              +influences[j][i]*influences[i][j]*(xSISJ[i][j] - xSIIJ[j][i] - xSIIJ[i][j] + xJIIJ[j][i]))
                                              +2.*gossip/Nt*g_jiij;

      opinions[nbStep+1][j][i]=opinions[nbStep][j][i] + 2./Nc*(influences[j][i]*(opinions[nbStep][i][ng] - opinions[nbStep][j][i]) + dInfluences[j][i]*(xJI2[j][i] - xSIJI[i][j] + xSISJ[i][j] - xSIIJ[j][i]))
                                                      + gossip*2./Nt*s_ji;
    }
    opinions[nbStep+1][i][ng]=opinions[nbStep][i][ng] + s_si;
    new_xSI2[0][i]=xSI2[i] + s_si2;
  }
  for(IdGroup i=0;i<ng;i++){
    new_xSI2[1][i]=new_xSI2[0][i]*(mu*mu+(1-mu*mu)/Ng) + new_xSISJ[0][i][i]*((1-mu*mu)*(Ng-1)/Ng);
    new_xSISJ[1][i][i]=new_xSISJ[0][i][i]*(mu*mu+(1-mu*mu)*(Ng-1)/Ng) + new_xSI2[0][i]*((1-mu*mu)/Ng);
    new_xSIIJ[1][i][i]=new_xSIIJ[0][i][i]*(mu*mu+(1-mu*mu)/Ng) + new_xSIJI[0][i][i]*(1-mu*mu)/Ng + new_xSIJK[0][i][i][i]*(1-mu*mu)*(Ng-2)/Ng;
    new_xJIIJ[1][i][i]=new_xJIIJ[0][i][i]*(mu*mu+(1-mu*mu)/(Ng*(Ng-1)))
                      +new_xJIJK[0][i][i][i]*(1-mu*mu)*(Ng-2)/(Ng*(Ng-1))
                      +new_xJIKI[0][i][i][i]*(1-mu*mu)*(Ng-2)/(Ng*(Ng-1))
                      +new_xJIKL[0][i][i][i][i]*(1-mu*mu)*(Ng-2)*(Ng-3)/(Ng*(Ng-1))
                      +new_xJI2[0][i][i]*(1-mu*mu)/(Ng*(Ng-1))
                      +2*new_xJIIK[0][i][i][i]*(1-mu*mu)*(Ng-2)/(Ng*(Ng-1));

    for(IdGroup j=0;j<ng;j++){
      Dij=(i==j);
      new_xSIJI[1][i][j]=new_xSIJI[0][i][j]*(mu*mu+(1-mu*mu)/Ng) + new_xSIJK[0][i][j][i]*(1-mu*mu)*(Ng-1-Dij)/Ng + Dij*new_xSIIJ[0][i][i]*(1-mu*mu)/Ng;
      new_xJI2[1][j][i]=new_xJI2[0][j][i]*(mu*mu+(1-mu*mu)/(Ng*(Ng-Dij)))
                        +new_xJIJK[0][j][i][i]*(1-mu*mu)*(Ng-1-Dij)/(Ng*(Ng-Dij))
                        +new_xJIKI[0][j][i][j]*(1-mu*mu)*(Ng-1-Dij)/(Ng*(Ng-Dij))
                        +new_xJIKL[0][j][i][j][i]*(1-mu*mu)*(Ng-1-Dij)*(Ng-1-2*Dij)/(Ng*(Ng-Dij))
                        +Dij*new_xJIIJ[0][i][i]*(1-mu*mu)/(Ng*(Ng-Dij))
                        +2*Dij*new_xJIIK[0][i][i][i]*(1-mu*mu)*(Ng-1-Dij)/(Ng*(Ng-Dij));
      new_xSIJK[1][i][j][i]=new_xSIJK[0][i][j][i]*(mu*mu+(1-mu*mu)*(Ng-1-Dij)/Ng)
                            +new_xSIJI[0][i][j]*(1-mu*mu)/Ng
                            +Dij*new_xSIIJ[0][i][i]*(1-mu*mu)/Ng;
      new_xJIJK[1][j][i][i]=new_xJIJK[0][j][i][i]*(mu*mu + (1-mu*mu)*(Ng-1-Dij)/(Ng*(Ng-Dij)))
                            +new_xJI2[0][j][i]*(1-mu*mu)/(Ng*(Ng-Dij))
                            +new_xJIKI[0][j][i][j]*(1-mu*mu)*(Ng-1-Dij)/(Ng*(Ng-Dij))
                            +new_xJIKL[0][j][i][j][i]*(1-mu*mu)*(Ng-1-Dij)*(Ng-1-2*Dij)/(Ng*(Ng-Dij))
                            +Dij*new_xJIIJ[0][i][i]*(1-mu*mu)/(Ng*(Ng-Dij))
                            +Dij*2*new_xJIIK[0][i][i][i]*(1-mu*mu)*(Ng-1-Dij)/(Ng*(Ng-Dij));
      new_xJIIK[1][j][i][i] = new_xJIIK[0][j][i][i]*(mu*mu+(1-mu*mu)*(Ng-Dij-1)/(Ng*(Ng-1)))
                              +new_xJIKI[0][j][i][i]*((1-mu*mu)*(Ng-1-Dij)/(Ng*(Ng-1)))
                              +new_xJIKL[0][j][i][i][i]*(1-mu*mu)*(Ng-1-Dij)*(Ng-2-Dij)/(Ng*(Ng-1))
                              +Dij*new_xJIIK[0][i][i][i]*(1-mu*mu)*(Ng-2)/(Ng*(Ng-1))
                              +Dij*new_xJI2[0][i][i]*(1-mu*mu)/(Ng*(Ng-1))
                              +Dij*new_xJIJK[0][i][i][i]*(1-mu*mu)*(Ng-2)/(Ng*(Ng-1))
                              +Dij*new_xJIIJ[0][i][i]*(1-mu*mu)/(Ng*(Ng-1));
      for(IdGroup p=0;p<ng;p++){
        Dip=(i==p);
        Djp=(j==p);
        new_xJIKI[1][j][i][p]=new_xJIKI[0][j][i][p]*(mu*mu+(1-mu*mu)*(Ng-Dip-Djp)/(Ng*(Ng-Dip)))
                              +new_xJIKL[0][j][i][p][i]*(1-mu*mu)*(Ng-Dip-Djp)*(Ng-Dij-Dip-1)/(Ng*(Ng-Dip))
                              +Dip*new_xJIIK[0][j][i][i]*(1-mu*mu)*(Ng-Dij-1)/(Ng*(Ng-Dij))
                              +Dij*new_xJIIK[0][p][i][i]*(1-mu*mu)*(Ng-Dip-1)/(Ng*(Ng-Dij))
                              +Djp*new_xJI2[0][j][i]*(1-mu*mu)/(Ng*(Ng-Dip))
                              +Djp*new_xJIJK[0][j][i][i]*(1-mu*mu)*(Ng-Dij-1)/(Ng*(Ng-Dip))
                              +Dij*Dip*new_xJIIJ[0][i][i]*(1-mu*mu)/(Ng*(Ng-1));
        new_xJIKL[1][j][i][p][i] = new_xJIKL[0][j][i][p][i]*(mu*mu+(1-mu*mu)*(Ng-Dip-Djp)*(Ng-Dij-Dip-1)/(Ng*(Ng-Dip)))
                              +new_xJIKI[0][j][i][p]*(1-mu*mu)*(Ng-Dip-Djp)/(Ng*(Ng-Dip))
                              +Dip*new_xJIIK[0][j][i][i]*(1-mu*mu)*(Ng-Dij-1)/(Ng*(Ng-Dip))
                              +Dij*new_xJIIK[0][p][i][i]*(1-mu*mu)*(Ng-Dip-Djp)/(Ng*(Ng-Dij))
                              +Djp*new_xJI2[0][j][i]*(1-mu*mu)/(Ng*(Ng-Dip))
                              +Djp*new_xJIJK[0][j][i][i]*(1-mu*mu)*(Ng-Dij-1)/(Ng*(Ng-Dip))
                              +Dij*Dip*new_xJIIJ[0][i][i]*(1-mu*mu)/(Ng*(Ng-1));

      }
    }
  }

  for(IdGroup i=0;i<ng;i++){
    equis[nbStep+1][i]=equis[nbStep][i];
    double Si=0;
    for(IdGroup j=0;j<ng;j++){
      Si+=(Ng-(i==j))*influences[i][j]/influences[j][i];
    }
    for(IdGroup j=0;j<ng;j++){
      equis[nbStep+1][i]+=(1./(1+Si))*2*(Ng-(i==j))/Nc*
        (dInfluences[i][j]*(xSIJI[i][j]-xSI2[i])+influences[i][j]/influences[j][i]*dInfluences[j][i]*(xJI2[j][i]-xSIJI[i][j]));
      effectByGroup[nbStep+1][i][j]=(1./(1+Si))*2*(Ng-(i==j))/Nc*
        (dInfluences[i][j]*(xSIJI[i][j]-xSI2[i])+influences[i][j]/influences[j][i]*dInfluences[j][i]*(xJI2[j][i]-xSIJI[i][j]));
      positiveBias[nbStep+1][i][j]+=(1./(1+Si))*2*(Ng-(i==j))/Nc*dInfluences[i][j]*(xSIJI[i][j]-xSI2[i]);
      negativeBias[nbStep+1][i][j]+=(1./(1+Si))*2*(Ng-(i==j))/Nc*influences[i][j]/influences[j][i]*dInfluences[j][i]*(xJI2[j][i]-xSIJI[i][j]);
      for(IdGroup p=0;p<ng;p++){
        gossipBias[nbStep+1][i][j]+=(1./(1+Si))*2*gossip*(Ng-(i==j))*(Ng-(p==i)-(p==j))/Nt*influences[i][j]/influences[j][i]*influences[j][p]*(opinions[nbStep][p][i]-opinions[nbStep][j][i]);
      }
    }
  }

  xSI2=new_xSI2[1];
  xJI2=new_xJI2[1];
  xSIJI=new_xSIJI[1];
  xJIKI=new_xJIKI[1];

  xSISJ=new_xSISJ[1];
  xSIIJ=new_xSIIJ[1];
  xJIIJ=new_xJIIJ[1];
  xSIJK=new_xSIJK[1];
  xJIJK=new_xJIJK[1];
  xJIIK=new_xJIIK[1];
  xJIKL=new_xJIKL[1];

  nbStep++;
}

void Approximation::writeOpinions(std::string fileName){
  std::ofstream myfile;
  myfile.open(fileName);
  myfile<<"step,rep(i),rep(j),self_i,x_ji"<<std::endl;

  for(unsigned int t=0;t<nbStep;t++){
    for(IdAgent i=0;i<nbGroups;i++){
      for(IdAgent j=0;j<nbGroups;j++){
        myfile<<t<<","<<startReputation[i]<<","<<startReputation[j]<<",";
        myfile<<opinions[t][i][nbGroups]<<","<<opinions[t][j][i]<<std::endl;
      }
    }
  }
  myfile.close();
}

//getters
double Approximation::getNoise2(){
  return pow(delta,2)/3;
}

double Approximation::getTrueOpinion(IdGroup i, IdGroup j){
  return opinions[nbStep][i][j]+startReputation[j];
}

double Approximation::getTrueSelfOpinion(IdGroup i){
  return opinions[nbStep][i][nbGroups]+startReputation[i];
}
