#include "Leviathan.h"
#include "Experiment.h"
#include <math.h>
#include <algorithm>
#include <iostream>
#include <fstream>
#include <climits>
#include <string>



float Leviathan::borne(float value){
    float ret=value;;
    if(!ignoreBorder){
        if(value>1)
            ret= 1;
        else if (value<-1)
            ret= -1;
    }
    return ret;
}

//-----------------------------------------------------------------Agent

void Agent::init(unsigned int nbAgents, unsigned int nbGroups){
    opinions=std::vector<float>(nbAgents,0);
    deltaOpinions=std::vector<float>(nbAgents,0);
    if(dady->gossip>0){
        for(IdGroup g=0;g<nbGroups;g++){
            gossipAbout.push_back(std::vector<IdAgent>());
            for(IdAgent i=0;i<nbAgents;i++){
              if(i!=id)
                  gossipAbout.back().push_back(i);
            }
        }
    }
}

IdAgent Agent::gossipToWho(){
    IdAgent Aq;
    IdGroup otherGroup=dady->agents[directInteraction].group;
    Aq=0;
    if(Aq!=0 || gossipAbout[otherGroup].size()==0 || countGossipAbout==gossipAbout[otherGroup].size()-1){
        Aq=UINT_MAX;
    }else{
        unsigned int pos;
        do{
            pos = std::uniform_int_distribution<int>(countGossipAbout, gossipAbout[otherGroup].size()-1)(g_m_t_generator);
            Aq=gossipAbout[otherGroup][pos];
            std::swap(gossipAbout[otherGroup][countGossipAbout],gossipAbout[otherGroup][pos]);
            countGossipAbout++;
        }while(Aq==directInteraction);
        dady->agents[directInteraction].listGossip.push_back(Aq);
    }
    return Aq;
}

void Agent::updateOpinions(){
    //opinions about agents
    opinions[directInteraction]=dady->borne(opinions[directInteraction]+deltaOpinions[directInteraction]);
    deltaOpinions[directInteraction]=0;
    opinions[id]=dady->borne(opinions[id]+deltaOpinions[id]);
    deltaOpinions[id]=0;
    for(unsigned int i=0;i<listGossip.size();i++){
      opinions[listGossip[i]]=dady->borne(opinions[listGossip[i]]+deltaOpinions[listGossip[i]]);
      deltaOpinions[listGossip[i]]=0;
    }
    listGossip.clear();
}

//------------------------------------------------------------------Leviathan

Leviathan::Leviathan(float noise_, unsigned int gossip_, float regroupFactor_):
    fixe_g_m_t_generator(),getRandomAgent_(),noise(noise_),gossip(gossip_),regroupFactor(regroupFactor_),
    nbStep(0),agents(),nbGroup(0),isInitialized(false),g_noise(-noise_, noise_),
    ignoreBorder(false),oneInterractionParStep(false)
    {
}

Leviathan::~Leviathan()
{
//    outfile.close();
}

void Leviathan::addGroup(unsigned int numberAgents,float sigma_){
    for(unsigned int i=0;i<numberAgents;i++)
        agents.push_back(Agent(this,nbGroup,getNbAgents(),sigma_));
    nbGroup++;
}

void Leviathan::init(std::vector<float> baseGroupsOpinions,bool useTheSeed){
    //init of agents (that contains opinions)
    nbStep=0;
    for(IdAgent i=0;i<getNbAgents();i++)
        agents[i].init(getNbAgents(),getNbGroups());

    getRandomAgent_=std::uniform_int_distribution<int>(0,getNbAgents()-1);
    if(useTheSeed)
        fixe_g_m_t_generator.seed(seed);
    else
        fixe_g_m_t_generator.seed(std::chrono::system_clock::now().time_since_epoch().count());
    getRandomAgent_.reset();

    isInitialized=true;
    startingOpinionsByGroup=std::vector<float>(nbGroup,0);
    if(baseGroupsOpinions.size()!=0){
      startingOpinionsByGroup=baseGroupsOpinions;
    }
    resetOpinions();
}

void Leviathan::resetOpinions(){
  for(IdAgent a=0;a<getNbAgents();a++){
    for(IdGroup a2=0;a2<getNbAgents();a2++)
      agents[a2].opinions[a]=startingOpinionsByGroup[agents[a].group];
  }
}

void Leviathan::step(){
    if(!isInitialized)
        init();
    nbStep ++;
    IdAgent Ai;
    IdAgent Aj;
    if(oneInterractionParStep==false){
      for(unsigned int i=0;i<getNbAgents();i++){
          selectCoupleForInterraction(Ai,Aj);
          directInteraction(Ai,Aj);
          directInteraction(Aj,Ai);
          gossipInteraction(Ai,Aj);
          agents[Ai].updateOpinions();
          agents[Aj].updateOpinions();
          agents[Ai].countGossipAbout=0;
          agents[Aj].countGossipAbout=0;
      }
    }else{
        selectCoupleForInterraction(Ai,Aj);
        directInteraction(Ai,Aj);
        directInteraction(Aj,Ai);
        gossipInteraction(Ai,Aj);
        agents[Ai].updateOpinions();
        agents[Aj].updateOpinions();
        agents[Ai].countGossipAbout=0;
        agents[Aj].countGossipAbout=0;
    }
    regroupOpinions();
}

void Leviathan::selectCoupleForInterraction(IdAgent & Ai,IdAgent & Aj){
    Ai=getRandomAgent();
    do{
        Aj=getRandomAgent();
    }while(Ai==Aj);
    agents[Ai].directInteraction=Aj;
    agents[Aj].directInteraction=Ai;
}

void Leviathan::directInteraction(IdAgent Ai,IdAgent Aj){
    float opinionAji=getOpinion(Aj,Ai);
    float opinionAjj=getOpinion(Aj,Aj);
    float p = 1. / (1. + exp((getOpinion(Ai,Ai)-getOpinion(Ai,Aj))/agents[Ai].sigma));
    agents[Ai].influenceOther = p;
    agents[Ai].deltaOpinions[Ai] += p * (opinionAji - getOpinion(Ai,Ai) + g_noise(g_m_t_generator));
    agents[Ai].deltaOpinions[Aj] += p * (opinionAjj - getOpinion(Ai,Aj) + g_noise(g_m_t_generator));
}

void Leviathan::gossipInteraction(IdAgent Ai,IdAgent Aj){
    for(unsigned int g=0;g<gossip;g++){
        oneGossip(Ai,Aj);
        oneGossip(Aj,Ai);
    }
}

void Leviathan::oneGossip(IdAgent Ai,IdAgent Aj){
    IdAgent Aq=agents[Aj].gossipToWho();
    if(Aq!=UINT_MAX){
        float p=agents[Ai].influenceOther;
        float opinionAjq = getOpinion(Aj,Aq);
        agents[Ai].deltaOpinions[Aq] += p * (opinionAjq - getOpinion(Ai,Aq)+ g_noise(g_m_t_generator) );
    }
}

void Leviathan::regroupOpinions(){
  for(IdGroup g1=0;g1<getNbGroups();g1++){
    for(IdGroup g2=0;g2<getNbGroups();g2++){
      reduceMinMaxDistance(g1,g2,regroupFactor);
    }
  }
}

void Leviathan::writeAllInfos(std::string file_name,unsigned int mode){
//    std::ofstream myfile;
//    std::ifstream ifile(file_name);
//    if (mode==0){
//        if(ifile.fail()){
//            ifile.close();
//            myfile.open(file_name);
//            myfile << "step,i,j,groupe i,groupe j, opinioni_j,opinioni_groupej,biais soi meme, biais son groupe, opinioni_j - opinion moyenne membres du meme groupe que j\n";
//        }else{
//            ifile.close();
//            myfile.open(file_name,std::ios::app);
//        }
//        for(IdAgent i=0;i<getNbAgents();i++){
//            for(IdAgent j=0;j<getNbAgents();j++){
//                myfile << nbStep <<","<<i<<","<<j<<","<<agents[i].group <<"," <<agents[j].group << "," << getOpinion(i,j)<<","<<getOpinionGroup(i,agents[j].group);
//                myfile<<"," <<getOpinion(i,i)-getAverageOpinionTo(i)<<","<< getOpinionGroup(i,agents[i].group)- getAverageOpinionToGroup(agents[i].group)<<","<< getOpinion(i,j)-getAverageOpinionToMemberOfGroup(agents[j].group)<<"\n";
//            }
//        }
//    }else if(mode==1){
//        if(ifile.fail()){
//            ifile.close();
//            myfile.open(file_name);
//            myfile << "step,i,selfGroup bias\n";
//        }else{
//            ifile.close();
//            myfile.open(file_name,std::ios::app);
//        }
//        for(IdAgent i=0;i<getNbAgents();i++){
//            myfile << nbStep <<","<<i<<"," <<getOpinion(i,i)-getAverageOpinionTo(i)<<"\n";
//        }
//    }
//    myfile.close();
}

void Leviathan::writeOpinions(std::string fileName){
    std::ofstream myfile;
    myfile.open(fileName);
    myfile << "Run,population,groupsNum,mu,sigma,noise_intensity,gossip";
    for(IdAgent i=0;i<getNbAgents();i++)
        myfile<<","<<"OPI"<<i;
    myfile<<"\n";
    myfile<<std::setprecision(2);
    for(IdAgent i=0;i<getNbAgents();i++){
        myfile<<nbStep<<","<<getNbAgents()<<","<<nbGroup<<","<<regroupFactor<<","<<agents[i].sigma<<","<<noise<<","<<gossip;
        for(IdAgent j=0;j<getNbAgents();j++)
            myfile<<","<<getOpinion(i,j);
        myfile<<"\n";
    }
    myfile.close();
}

void Leviathan::reduceMinMaxDistance(IdGroup from,IdGroup to, float reduceFactor){
  float mid =0;
  float nb=0;
  float midSelf =0;
  float nbSelf=0;
  for(unsigned int i=0;i<getNbAgents();i++){
    if(agents[i].group==from){
      for(unsigned int j=0;j<getNbAgents();j++){
        if(agents[j].group==to){
          if(i==j){
            midSelf+=agents[i].opinions[j];
            nbSelf++;
          }else{
            mid+=agents[i].opinions[j];
            nb++;
          }
        }
      }
    }
  }
  mid=mid/nb;
  midSelf=midSelf/nbSelf;
  for(unsigned int i=0;i<getNbAgents();i++){
    if(agents[i].group==from){
      for(unsigned int j=0;j<getNbAgents();j++){
        if(agents[j].group==to){
          if(i==j){
            agents[i].opinions[j]=reduceFactor*agents[i].opinions[j]+(1-reduceFactor)*midSelf;
          }else{
            agents[i].opinions[j]=reduceFactor*agents[i].opinions[j]+(1-reduceFactor)*mid;
          }
        }
      }
    }
  }
}

//GETTERS
